from .g2pk3 import G2p
import unicodedata
import re


g2p = G2p()

repl_dict = {
    'ᄁ': 'ᄏ',
    'ᄄ': 'ᄐ',
    'ᄈ': 'ᄑ',
    'ᄊ': 'ᄉ',
    'ᄍ': 'ᄎ',
    'ᅢ': 'ᅦ',
    'ᅤ': 'ᅨ',
    'ᅥ': 'ᅩ',
    'ᅧ': 'ᅭ',
    'ᅫ': 'ᅬ',
    'ᅰ': 'ᅬ',
    'ᅳ': 'ᅮ',
    'ᅴ': 'ᅱ',

    '가': 'ガ',
    '갸': 'ギャ',
    '게': 'ゲ',
    '계': 'ギェ',
    '고': 'ゴ',
    '과': 'グァ',
    '괴': 'グェ',
    '교': 'ギョ',
    '구': 'グ',
    '궈': 'グォ',
    '귀': 'グィ',
    '규': 'ギュ',
    '기': 'ギ',

    '나': 'ナ',
    '냐': 'ニャ',
    '네': 'ネ',
    '녜': 'ニェ',
    '노': 'ノ',
    '놔': 'ヌァ',
    '뇌': 'ヌェ',
    '뇨': 'ニョ',
    '누': 'ヌ',
    '눠': 'ヌォ',
    '뉘': 'ヌィ',
    '뉴': 'ニュ',
    '니': 'ニ',

    '다': 'ダ',
    '댜': 'デャ',
    '데': 'デ',
    '뎨': 'ディェ',
    '도': 'ド',
    '돠': 'ドゥァ',
    '되': 'ドゥェ',
    '됴': 'デョ',
    '두': 'ドゥ',
    '둬': 'ドゥォ',
    '뒤': 'ドゥィ',
    '듀': 'デュ',
    '디': 'ディ',

    '라': 'ラ',
    '랴': 'リャ',
    '레': 'レ',
    '례': 'リェ',
    '로': 'ロ',
    '롸': 'ルァ',
    '뢰': 'ルェ',
    '료': 'リョ',
    '루': 'ル',
    '뤄': 'ルォ',
    '뤼': 'ルィ',
    '류': 'リュ',
    '리': 'リ',

    '마': 'マ',
    '먀': 'ミャ',
    '메': 'メ',
    '몌': 'ミェ',
    '모': 'モ',
    '뫄': 'ムァ',
    '뫼': 'ムェ',
    '묘': 'ミョ',
    '무': 'ム',
    '뭐': 'ムォ',
    '뮈': 'ムィ',
    '뮤': 'ミュ',
    '미': 'ミ',

    '바': 'バ',
    '뱌': 'ビャ',
    '베': 'ベ',
    '볘': 'ビェ',
    '보': 'ボ',
    '봐': 'ブァ',
    '뵈': 'ブェ',
    '뵤': 'ビョ',
    '부': 'ブ',
    '붜': 'ブォ',
    '뷔': 'ブィ',
    '뷰': 'ビュ',
    '비': 'ビ',

    '사': 'サ',
    '샤': 'シャ',
    '세': 'セ',
    '셰': 'シェ',
    '소': 'ソ',
    '솨': 'スァ',
    '쇠': 'スェ',
    '쇼': 'ショ',
    '수': 'ス',
    '숴': 'スォ',
    '쉬': 'スゥィ',
    '슈': 'シュ',
    '시': 'シ',

    '아': 'ア',
    '야': 'ヤ',
    '에': 'エ',
    '예': 'イェ',
    '오': 'オ',
    '와': 'ワ',
    '외': 'ウェ',
    '요': 'ヨ',
    '우': 'ウ',
    '워': 'ウォ',
    '위': 'ウィ',
    '유': 'ユ',
    '이': 'イ',

    '자': 'ザ',
    '쟈': 'ジャ',
    '제': 'ゼ',
    '졔': 'ジェ',
    '조': 'ゾ',
    '좌': 'ズァ',
    '죄': 'ズェ',
    '죠': 'ジョ',
    '주': 'ズ',
    '줘': 'ズォ',
    '쥐': 'ズィ',
    '쥬': 'ジュ',
    '지': 'ジ',

    '차': 'シァ',
    '챠': 'チァ',
    '체': 'ツェ',
    '쳬': 'チェ',
    '초': 'ツォ',
    '촤': 'チュァ',
    '최': 'チュェ',
    '쵸': 'チォ',
    '추': 'ツ',
    '춰': 'チュォ',
    '취': 'チュィ',
    '츄': 'チュ',
    '치': 'チ',

    '카': 'カ',
    '캬': 'キャ',
    '케': 'ケ',
    '켸': 'キェ',
    '코': 'コ',
    '콰': 'クァ',
    '쾨': 'クェ',
    '쿄': 'キョ',
    '쿠': 'ク',
    '쿼': 'クォ',
    '퀴': 'クィ',
    '큐': 'キュ',
    '키': 'キ',

    '타': 'タ',
    '탸': 'テャ',
    '테': 'テ',
    '톄': 'ティェ',
    '토': 'ト',
    '톼': 'トァ',
    '퇴': 'トェ',
    '툐': 'テョ',
    '투': 'トゥ',
    '퉈': 'トォ',
    '튀': 'トィ',
    '튜': 'テュ',
    '티': 'ティ',

    '파': 'パ',
    '퍄': 'ピャ',
    '페': 'ペ',
    '폐': 'ピェ',
    '포': 'ポ',
    '퐈': 'プァ',
    '푀': 'プェ',
    '표': 'ピョ',
    '푸': 'プ',
    '풔': 'プォ',
    '퓌': 'プィ',
    '퓨': 'ピュ',
    '피': 'ピ',

    '하': 'ハ',
    '햐': 'ヒャ',
    '헤': 'ヘ',
    '혜': 'ヒェ',
    '호': 'ホ',
    '화': 'ファ',
    '회': 'フェ',
    '효': 'ヒョ',
    '후': 'フ',
    '훠': 'フォ',
    '휘': 'フィ',
    '휴': 'ヒュ',
    '히': 'ヒ',

    'ᆨ': 'ッー',
    'ᆫ': 'ン',
    'ᆮ': 'ッ',
    'ᆯ': 'ー',
    'ᆷ': 'ンー',
    'ᆸ': 'ッー',
    'ᆼ': 'ンー'
}

def korean2katakana(text: str):
    text = unicodedata.normalize('NFD', g2p(text))
    for h, k in repl_dict.items():
        text = text.replace(h, k)
    return unicodedata.normalize('NFKC', text)


def toKana(text: str):
    return re.sub(
        "[가-힣a-zA-Z]+", 
        lambda x: korean2katakana(x.group(0)), 
        text
    )

if __name__ == '__main__':
    print(toKana('Hello. こんいちわ。안녕하세요.')) # ホーロ. こんいちわ。アンニョンーハセヨ.